     /**
      * \brief JSON Array Implementation
      *
      * The value of a JSON object can be an array. This service program deals
      * with such arrays. Arrays themselves can also contain arrays. An array
      * can only contain values and arrays but no key/value pairs.
      *
      * <br><br>
      *
      * Throughout this service program is the index always 0-based. Which
      * means that the first entry has the index 0, the second entry has the
      * index 1 and so on.
      *
      * <br><br>
      *
      * If the index is out of the bounds of the array an escape message will
      * be send. If a procedure for the retrieval of a special type of value
      * is called on an entry of another type an escape message will be send
      * too.
      *
      * \author Mihael Schmidt
      * \date   19.04.2008
      *
      * \link http://www.json.org JSON
      *
      * \rev 07.03.2009 Mihael Schmidt
      *      Added <em>jsona_size</em> procedure.
      */

      *-------------------------------------------------------------------------
      * Copyright (c) 2007-2009 Mihael Schmidt
      * All rights reserved.
      *
      * This file is part of the JSON service program.
      *
      * JSON service program is free software: you can redistribute it and/or modify it under
      * the terms of the GNU Lesser General Public License as published by
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * JSON service program is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU Lesser General Public License for more details.
      *
      * You should have received a copy of the GNU Lesser General Public
      * License along with JSON service program.  If not, see http://www.gnu.org/licenses/.
      *-------------------------------------------------------------------------

     H nomain
     H bnddir('QC2LE')
     H copyright('Copyright (c) 2007-2009 Mihael Schmidt. All rights reserved.')


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D memcpy          PR                  extproc('memcpy')
     D   source                        *   value
     D   dest                          *   value
     D   length                      10U 0 value
      *
     D memcmp          PR            10I 0 extproc('memcmp')
     D   buffer1                       *   value
     D   buffer2                       *   value
     D   length                      10U 0 value
      *
     D strlen          PR            10I 0 extproc('strlen')
     D   buffer1                       *   value

      /copy JSON_H
      /copy JSON_INT_H
      /copy LLIST_H
      /copy CEEAPI_H


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
      /copy JSON_C


      *-------------------------------------------------------------------------
      * Global Variables
      *-------------------------------------------------------------------------
     D hex_null        S              1A   inz(x'00')
      *
      * Parameters passed to CEEDOD
      *
     D descType        S             10I 0
     D dataType        S             10I 0
     D descInfo1       S             10I 0
     D descInfo2       S             10I 0
     D length          S             10I 0
      *
     D tmpl_entry      DS                  qualified
     D   type                        10I 0
     D   x                        65531A
     D   string                   65531A   overlay(x)
     D   boolean                       N   overlay(x)
     D   integer                     10I 0 overlay(x)
     D   long                        20I 0 overlay(x)
     D   double                       8F   overlay(x)


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------

     /**
      * \brief Create JSON array
      *
      * Creates a json array "object". Currently a linked list is used
      * as a backend for the json array.
      *
      * \author Mihael Schmidt
      * \date   19.04.2008
      *
      * \return Pointer to the JSON array
      */
     P jsona_create    B                   export
     D                 PI              *
      /free
       return list_create();
      /end-free
     P                 E

     /**
      * \brief Dispose JSON array
      *
      * Disposes a JSON array and all values it contains.
      * All added arrays are also disposed.
      *
      * \author Mihael Schmidt
      * \date   19.04.2008
      *
      * \param Pointer to the JSON array
      */
     P jsona_dispose   B                   export
     D                 PI
     D   jsonArrPtr                    *
      *
     D tmpPtr          S               *
     D tmpDs           DS                  likeds(tmpl_entry) based(tmpPtr)
     D tmpPtrDs        DS                  qualified based(tmpPtr)
     D   type                        10I 0
     D   ptr                           *
      /free
       tmpPtr = list_getNext(jsonArrPtr);
       dow (tmpPtr <> *null);
         if (tmpDs.type = TYPE_ARRAY);
           jsona_dispose(tmpPtrDs.ptr);
         elseif (tmpDs.type = TYPE_OBJECT);
           json_dispose(tmpPtrDs.ptr);
         endif;

         tmpPtr = list_getNext(jsonArrPtr);
       enddo;

       list_dispose(jsonArrPtr);
      /end-free
     P                 E


     /**
      * \brief Remove an entry from the array
      *
      * \param Pointer to the JSON array
      * \param Index of the element to be removed (0-based)
      */
     P jsona_remove    B                   export
     D                 PI                  opdesc
     D   jsonArrPtr                    *   const
     D   index                       10I 0 const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  likeds(tmpl_entry) based(tmpPtr)
     D ptr             S               *
     D tmpPtrDs        DS                  qualified based(tmpPtr)
     D   type                        10I 0
     D   array                         *
      /free
       tmpPtr = list_get(jsonArrPtr : index);
       if (tmpPtr <> *null and tmpDs.type = TYPE_ARRAY);
         jsona_dispose(tmpPtrDs.array);
       endif;

       list_remove(jsonArrPtr : index);
      /end-free
     P                 E


     /**
      * \brief Clear array
      *
      * Removes all elements from the array.
      *
      * \param Pointer to the JSON array
      */
     P jsona_clear     B                   export
     D                 PI
     D   jsonArrPtr                    *   const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  likeds(tmpl_entry) based(tmpPtr)
     D ptr             S               *
     D tmpPtrDs        DS                  qualified based(tmpPtr)
     D   type                        10I 0
     D   array                         *
      /free
       tmpPtr = list_getNext(jsonArrPtr);
       dow (tmpPtr <> *null);
         if (tmpDs.type = TYPE_ARRAY);
           jsona_dispose(tmpPtrDs.array);
         endif;
         tmpPtr = list_getNext(jsonArrPtr);
       enddo;

       list_clear(jsonArrPtr);
      /end-free
     P                 E

     /**
      * \brief Add character entry
      *
      * Adds an entry of type character.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Value
      * \param Index (default: last)
      */
     P jsona_putString...
     P                 B                   export
     D                 PI                  opdesc
     D   jsonArrPtr                    *   const
     D   value                    65535A   const options(*varsize)
     D   index                       10U 0 const options(*nopass)
      /free
       cee_getOpDescInfo(2 : descType : dataType : descInfo1 : descInfo2 :
                         length : *omit);
       tmpl_entry.type = TYPE_STRING;
       tmpl_entry.string = %subst(value : 1 : length);

       if (%parms() = 2);
         list_add(jsonArrPtr : %addr(tmpl_entry) : length + 4);
       else;
         list_add(jsonArrPtr : %addr(tmpl_entry) : length + 4 : index);
       endif;
      /end-free
     P                 E


     /**
      * \brief Add boolean entry
      *
      * Adds an entry of type boolean.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Value
      * \param Index (default: last)
      */
     P jsona_putBoolean...
     P                 B                   export
     D                 PI
     D   jsonArrPtr                    *   const
     D   value                         N   const
     D   index                       10U 0 const options(*nopass)
      /free
       tmpl_entry.type = TYPE_BOOLEAN;
       tmpl_entry.boolean = value;

       if (%parms() = 2);
         list_add(jsonArrPtr : %addr(tmpl_entry) : 5);
       else;
         list_add(jsonArrPtr : %addr(tmpl_entry) : 5 : index);
       endif;
      /end-free
     P                 E


     /**
      * \brief Add integer entry
      *
      * Adds an entry of type integer.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Value
      * \param Index (default: last)
      */
     P jsona_putInt...
     P                 B                   export
     D                 PI
     D   jsonArrPtr                    *   const
     D   value                       10I 0 const
     D   index                       10U 0 const options(*nopass)
      /free
       tmpl_entry.type = TYPE_INT;
       tmpl_entry.integer = value;

       if (%parms() = 2);
         list_add(jsonArrPtr : %addr(tmpl_entry) : 8);
       else;
         list_add(jsonArrPtr : %addr(tmpl_entry) : 8 : index);
       endif;
      /end-free
     P                 E


     /**
      * \brief Add long entry
      *
      * Adds an entry of type long.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Value
      * \param Index (default: last)
      */
     P jsona_putLong...
     P                 B                   export
     D                 PI
     D   jsonArrPtr                    *   const
     D   value                       20I 0 const
     D   index                       10U 0 const options(*nopass)
      /free
       tmpl_entry.type = TYPE_LONG;
       tmpl_entry.long = value;

       if (%parms() = 2);
         list_add(jsonArrPtr : %addr(tmpl_entry) : 12);
       else;
         list_add(jsonArrPtr : %addr(tmpl_entry) : 12 : index);
       endif;
      /end-free
     P                 E


     /**
      * \brief Add double entry
      *
      * Adds an entry of type double.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Value
      * \param Index (default: last)
      */
     P jsona_putDouble...
     P                 B                   export
     D                 PI
     D   jsonArrPtr                    *   const
     D   value                        8F   const
     D   index                       10U 0 const options(*nopass)
      /free
       tmpl_entry.type = TYPE_DOUBLE;
       tmpl_entry.double = value;

       if (%parms() = 2);
         list_add(jsonArrPtr : %addr(tmpl_entry) : 12);
       else;
         list_add(jsonArrPtr : %addr(tmpl_entry) : 12 : index);
       endif;
      /end-free
     P                 E


     /**
      * \brief Add array entry
      *
      * Adds an entry of type array.
      *
      * \author Mihael Schmidt
      * \date   19.04.2008
      *
      * \param Pointer to the JSON array
      * \param Array
      * \param Index (default: last)
      */
     P jsona_putArray...
     P                 B                   export
     D                 PI
     D   jsonArrPtr                    *   const
     D   value                         *   const
     D   index                       10U 0 const options(*nopass)
      *
     D tmpDs           DS                  qualified
     D   type                        10I 0
     D   array                         *
      /free
       tmpDs.type = TYPE_ARRAY;
       tmpDs.array = value;

       if (%parms() = 2);
         list_add(jsonArrPtr : %addr(tmpDs) : %size(tmpDs));
       else;
         list_add(jsonArrPtr : %addr(tmpDs) : %size(tmpDs) : index);
       endif;
      /end-free
     P                 E


     /**
      * \brief Get string entry
      *
      * Returns a pointer to the value of the entry for the passed key.
      * The value is null terminated and can be processed with the %str BIF.
      *
      * <br><br>
      *
      * If the entry is not of type string an escape message will be send.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Index
      *
      * \return Pointer to a null-terminated string
      */
     P jsona_getString...
     P                 B                   export
     D                 PI              *
     D   jsonArrPtr                    *   const
     D   index                       10U 0 const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  likeds(tmpl_entry) based(tmpPtr)
      /free
       tmpPtr = list_get(jsonArrPtr : index);
       if (tmpPtr <> *null);
         if (tmpDs.type = TYPE_STRING);
           return tmpPtr + 4;
         else;
           sendEscapeMessage(MSG_ENTRY_TYPE_MISMATCH);
         endif;
       else;
         sendEscapeMessage(MSG_ARRAY_ELEMENT_NOT_FOUND);
       endif;

       return *null; // dummy => will never come to this point
      /end-free
     P                 E


     /**
      * \brief Get boolean entry
      *
      * Returns the boolean value of the entry.
      *
      * <br><br>
      *
      * If the entry is not of type boolean an escape message will be send.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Index
      *
      * \return Boolean value of the entry
      */
     P jsona_getBoolean...
     P                 B                   export
     D                 PI              N
     D   jsonArrPtr                    *   const
     D   index                       10U 0 const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  likeds(tmpl_entry) based(tmpPtr)
      /free
       tmpPtr = list_get(jsonArrPtr : index);
       if (tmpPtr <> *null);
         if (tmpDs.type = TYPE_BOOLEAN);
           return tmpDs.boolean;
         else;
           sendEscapeMessage(MSG_ENTRY_TYPE_MISMATCH);
         endif;
       else;
         sendEscapeMessage(MSG_ARRAY_ELEMENT_NOT_FOUND);
       endif;

       return *off; // dummy => will never come to this point
      /end-free
     P                 E


     /**
      * \brief Get integer entry
      *
      * Returns the integer value of the entry.
      *
      * <br><br>
      *
      * If the entry is not of type integer an escape message will be send.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Index
      *
      * \return Integer value of the entry
      */
     P jsona_getInt...
     P                 B                   export
     D                 PI            10I 0
     D   jsonArrPtr                    *   const
     D   index                       10U 0 const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  likeds(tmpl_entry) based(tmpPtr)
      /free
       tmpPtr = list_get(jsonArrPtr : index);
       if (tmpPtr <> *null);
         if (tmpDs.type = TYPE_INT);
           return tmpDs.integer;
         else;
           sendEscapeMessage(MSG_ENTRY_TYPE_MISMATCH);
         endif;
       else;
         sendEscapeMessage(MSG_ARRAY_ELEMENT_NOT_FOUND);
       endif;

       return -1; // dummy => will never come to this point
      /end-free
     P                 E


     /**
      * \brief Get long entry
      *
      * Returns the long value of the entry.
      *
      * <br><br>
      *
      * If the entry is not of type long an escape message will be send.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Index
      *
      * \return Long value of the entry
      */
     P jsona_getLong...
     P                 B                   export
     D                 PI            20I 0
     D   jsonArrPtr                    *   const
     D   index                       10U 0 const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  likeds(tmpl_entry) based(tmpPtr)
      /free
       tmpPtr = list_get(jsonArrPtr : index);
       if (tmpPtr <> *null);
         if (tmpDs.type = TYPE_LONG);
           return tmpDs.long;
         else;
           sendEscapeMessage(MSG_ENTRY_TYPE_MISMATCH);
         endif;
       else;
         sendEscapeMessage(MSG_ARRAY_ELEMENT_NOT_FOUND);
       endif;

       return -1; // dummy => will never come to this point
      /end-free
     P                 E


     /**
      * \brief Get double entry
      *
      * Returns the double value of the entry.
      *
      * <br><br>
      *
      * If the entry is not of type double an escape message will be send.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Index
      *
      * \return Double value of the entry
      */
     P jsona_getDouble...
     P                 B                   export
     D                 PI             8F
     D   jsonArrPtr                    *   const
     D   index                       10U 0 const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  likeds(tmpl_entry) based(tmpPtr)
      /free
       tmpPtr = list_get(jsonArrPtr : index);
       if (tmpPtr <> *null);
         if (tmpDs.type = TYPE_DOUBLE);
           return tmpDs.double;
         else;
           sendEscapeMessage(MSG_ENTRY_TYPE_MISMATCH);
         endif;
       else;
         sendEscapeMessage(MSG_ARRAY_ELEMENT_NOT_FOUND);
       endif;

       return -1.0; // dummy => will never come to this point
      /end-free
     P                 E


     /**
      * \brief Get array entry
      *
      * Returns the array value of the entry.
      *
      * <br><br>
      *
      * If the entry is not of type array an escape message will be send.
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      * \param Index
      *
      * \return Pointer to the JSON array
      */
     P jsona_getArray...
     P                 B                   export
     D                 PI              *
     D   jsonArrPtr                    *   const
     D   index                       10U 0 const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  qualified based(tmpPtr)
     D   type                        10I 0
     D   array                         *
      /free
       tmpPtr = list_get(jsonArrPtr : index);
       if (tmpPtr <> *null);
         if (tmpDs.type = TYPE_ARRAY);
           return tmpDs.array;
         else;
           sendEscapeMessage(MSG_ENTRY_TYPE_MISMATCH);
         endif;
       else;
         sendEscapeMessage(MSG_ARRAY_ELEMENT_NOT_FOUND);
       endif;

       return *null; // dummy => will never come to this point
      /end-free
     P                 E


     /**
      * \brief String representation of the JSON array in JSON syntax
      *
      * Creates a null terminated string representation of the JSON array in
      * JSON syntax.
      *
      * <br><br>
      *
      * For entries of type string double quotes and backslashes will be escaped
      * (" => \" and \ => \\).
      *
      * \author Mihael Schmidt
      * \date   20.03.2008
      *
      * \param Pointer to the JSON array
      */
     P jsona_toString...
     P                 B                   export
     D                 PI              *
     D   jsonArrPtr                    *   const
      *
     D d_quote         S              2A
     D l_brace         S              3A
     D r_brace         S              3A
     D l_bracket       S              3A
     D r_bracket       S              3A
     D comma           S              4A
     D colon           S              2A
     D true            S              5A
     D false           S              6A
     D null_str        S              5A
      *
     D json_string_ptr...
     D                 S               *
     D json_string_length...
     D                 S             10I 0
     D tokenList       S               *
     D tmpString       S            310A
     D tmpLength       S             10I 0
     D entryPtr        S               *
     D entry           DS                  likeds(tmpl_entry) based(entryPtr)
     D tmpDs           DS                  qualified based(entryPtr)
     D   type                        10I 0
     D   ptr                           *
     D ptr             S               *
     D x               S             10I 0
      /free
       exsr init;

       // initialization
       //  - write start tag
       //  - init token list
       tokenList = list_create();
       list_add(tokenList : %addr(l_bracket) : %size(l_bracket));

       // process entries
       entryPtr = list_getNext(jsonArrPtr);
       dow (entryPtr <> *null);

         select;
           when (entry.type = TYPE_STRING);
             list_add(tokenList : %addr(d_quote) : %size(d_quote));
             list_add(tokenList : %addr(entry.string) :
                      strlen(%addr(entry.string)));
             escapeLastToken(tokenList);
             list_add(tokenList : %addr(d_quote) : %size(d_quote));

           when (entry.type = TYPE_INT);
             tmpString = %char(entry.integer);
             tmpLength = %len(%trimr(tmpString));
             list_add(tokenList : %addr(tmpString) : tmpLength);

           when (entry.type = TYPE_LONG);
             tmpString = %char(entry.long);
             tmpLength = %len(%trimr(tmpString));
             list_add(tokenList : %addr(tmpString) : tmpLength);

           when (entry.type = TYPE_DOUBLE);
             tmpString = %char(entry.double);
             tmpLength = %len(%trimr(tmpString));
             // JSON syntax expects a . as delimiter (and not ,)
             tmpString = %xlate(',' : '.' : tmpString);
             list_add(tokenList : %addr(tmpString) : tmpLength);

           when (entry.type = TYPE_BOOLEAN);
             if (entry.boolean);
               list_add(tokenList : %addr(true) : %size(true));
             else;
               list_add(tokenList : %addr(false) : %size(false));
             endif;

           when (entry.type = TYPE_ARRAY);
             ptr = jsona_toString(tmpDs.ptr);
             list_add(tokenList : ptr : strlen(ptr));
             dealloc(n) ptr;

           when (entry.type = TYPE_NULL);
             list_add(tokenList : %addr(entry.string) : 4);

           when (entry.type = TYPE_OBJECT);
             ptr = json_toString(tmpDs.ptr);
             list_add(tokenList : ptr : strlen(ptr));
             dealloc(n) ptr;

         endsl;


         // add comma if it is not the last entry
         if (entryPtr <> list_getLast(jsonArrPtr));
           list_add(tokenList : %addr(comma) : %size(comma));
         endif;

         // next entry
         entryPtr = list_getNext(jsonArrPtr);
       enddo;

       // write end tag
       list_add(tokenList : %addr(r_bracket) : %size(r_bracket));

       // calculate json string length
       ptr = list_getNext(tokenList);
       dow (ptr <> *null);
         json_string_length += strlen(ptr);
         ptr = list_getNext(tokenList);
       enddo;
       // +1 for hex Null
       json_string_length += 1;

       // allocate memory for json string
       json_string_ptr = %alloc(json_string_length);

       // build json string
       ptr = list_getNext(tokenList);
       dow (ptr <> *null);
         tmpLength = strlen(ptr);
         memcpy(json_string_ptr + x : ptr : tmpLength);
         x += tmpLength;
         ptr = list_getNext(tokenList);
       enddo;

       // append hex null
       memcpy(json_string_ptr + x : %addr(hex_null) : 1);

       // clean up
       if (tokenList <> *null);
         list_dispose(tokenList);
       endif;

       return json_string_ptr;

       begsr init;
         d_quote = '"' + hex_null;
         l_brace = '{ ' + hex_null;
         r_brace = ' }' + hex_null;
         l_bracket = '[ ' + hex_null;
         r_bracket = ' ]' + hex_null;
         colon = ':' + hex_null;
         comma = ' , ' + hex_null;
         true = 'true' + hex_null;
         false = 'false' + hex_null;
         null_str = 'null' + hex_null;
       endsr;
      /end-free
     P                 E


     /**
      * \brief Add null value entry
      *
      * Adds an entry of type null to the array.
      *
      * \author Mihael Schmidt
      * \date   11.05.2008
      *
      * \param Pointer to the JSON array
      * \param Index (default: last)
      */
     P jsona_putNull   B                   export
     D                 PI
     D   jsonArrPtr                    *   const
     D   index                       10U 0 const options(*nopass)
      /free
       tmpl_entry.type = TYPE_NULL;
       tmpl_entry.string = 'null';

       if (%parms() = 1);
         list_add(jsonArrPtr : %addr(tmpl_entry) : 4 + 4);
       else;
         list_add(jsonArrPtr : %addr(tmpl_entry) : 4 + 4 : index);
       endif;
      /end-free
     P                 E


     /**
      * \brief Add JSON object entry
      *
      * Adds an entry of type object to the array.
      *
      * \author Mihael Schmidt
      * \date   11.05.2008
      *
      * \param Pointer to the JSON array
      * \param Pointer to the JSON object
      * \param Index (default: last)
      */
     P jsona_putObject...
     P                 B                   export
     D                 PI
     D   jsonArrPtr                    *   const
     D   objPtr                        *   const
     D   index                       10U 0 const options(*nopass)
     D tmpDs           DS                  qualified
     D   type                        10I 0
     D   object                        *
      /free
       tmpDs.type = TYPE_OBJECT;
       tmpDs.object = objPtr;

       if (%parms() = 2);
         list_add(jsonArrPtr : %addr(tmpDs) : %size(tmpDs));
       else;
         list_add(jsonArrPtr : %addr(tmpDs) : %size(tmpDs) : index);
       endif;
      /end-free
     P                 E


     /**
      * \brief Get object entry
      *
      * Returns a pointer to the object.
      *
      * <br><br>
      *
      * If the entry is not of type object an escape message will be send.
      *
      * \author Mihael Schmidt
      * \date   11.05.2008
      *
      * \param Pointer to the JSON array
      * \param Index
      *
      * \return Pointer to the JSON object
      */
     P jsona_getObject...
     P                 B                   export
     D                 PI              *
     D   jsonArrPtr                    *   const
     D   index                       10U 0 const
      *
     D tmpPtr          S               *
     D tmpDs           DS                  qualified based(tmpPtr)
     D   type                        10I 0
     D   object                        *
      /free
       tmpPtr = list_get(jsonArrPtr : index);
       if (tmpPtr <> *null);
         if (tmpDs.type = TYPE_OBJECT);
           return tmpDs.object;
         else;
           sendEscapeMessage(MSG_ENTRY_TYPE_MISMATCH);
         endif;
       else;
         sendEscapeMessage(MSG_ARRAY_ELEMENT_NOT_FOUND);
       endif;

       return *null; // dummy => will never come to this point
      /end-free
     P                 E


     /**
      * \brief Get array size
      *
      * Returns the number of elements in the array.
      *
      * \param Pointer to the json array
      *
      * \return number of elements
      */
     P jsona_size      B                   export
     D                 PI            10U 0
     D   jsonArrPtr                    *   const
      /free
       return list_size(jsonArrPtr);
      /end-free
     P                 E
