     /**
      * \brief JSON Parser
      *
      * A utility module for parsing JSON strings.
      *
      * <br><br>
      *
      * The parser string should be a valid JSON string or else and escape
      * message will be sent to the caller.
      *
      * <br><br>
      *
      * The parser also accepts the equal sign (=) as a separator for the key
      * and the value. The parser also supports the ; instead of the , for
      * separating pairs of values or values in an array. Strings may be
      * enclosed in single or double quotes. Arrays can either be enclosed in
      * curly braces () or in brackets []. Empty objects and arrays are supported.
      * Unicode representation of a character is not supported like \u00CF.
      *
      * \author Mihael Schmidt
      * \date   2009-03-07
      *
      * \link http://www.json.org JSON.org
      *
      * \info The JSON syntax is not fully supported at the moment.
      */

      *-------------------------------------------------------------------------
      * Copyright (c) 2007-2009 Mihael Schmidt
      * All rights reserved.
      *
      * This file is part of the JSON service program.
      *
      * JSON service program is free software: you can redistribute it and/or modify it under
      * the terms of the GNU Lesser General Public License as published by
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * JSON service program is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU Lesser General Public License for more details.
      *
      * You should have received a copy of the GNU Lesser General Public
      * License along with JSON service program.  If not, see http://www.gnu.org/licenses/.
      *-------------------------------------------------------------------------


     H nomain
     H bnddir('QC2LE')
     H copyright('Copyright (c) 2007-2009 Mihael Schmidt. All rights reserved.')


      *---------------------------------------------------------------
      * Constants
      *---------------------------------------------------------------
     D CR              C                   const(x'0d')
     D LF              C                   const(x'25')
     D QUOTE           C                   const('''')
     D DOUBLE_QUOTE    C                   const('"')
     D NULL            C                   const(x'00')
     D TYPE_NO_VALUE   C                   const(-1)
      *
      /copy JSON_C


      *---------------------------------------------------------------
      * Prototypes
      *---------------------------------------------------------------
     D nextValue       PR
     D   retValPtr                     *   const
     D   header                        *
     D   dataType                    10I 0
      *
     D nextCleanChar   PR             1A
     D   header                        *
      *
     D nextString      PR         10000A
     D   header                        *
     D   quote                        1A   const
      *
     D nextArray       PR              *
     D   header                        *
      *
     D next            PR             1A
     D   header                        *
      *
     D back            PR
     D   header                        *
      *
     D more            PR              N
     D   header                        *
      *
     D abortParsing    PR
     D   header                        *
     D   message                   1000A   const
      *
     D strlen          PR            10I 0 extproc('strlen')
     D   buffer1                       *   value
      *
     D strcasecmp      PR            10I 0 extproc('strcasecmp')
     D   buffer1                       *   value
     D   buffer2                       *   value
      *
      /copy JSON_H


      *---------------------------------------------------------------
      * Data structures
      *---------------------------------------------------------------
     D tmpl_parser_ds  DS                  qualified based(nullPointer)
     D   json_string_ptr...
     D                                 *
     D   index_pos                   10I 0
      *
     D tmpl_return_value_ds...
     D                 DS                  qualified based(nullPointer)
     D   text                     65535A
     D   boolean                       N   overlay(text : 1)
     D   integer                     10I 0 overlay(text : 1)
     D   long                        20I 0 overlay(text : 1)
     D   double                       8F   overlay(text : 1)
     D   pointer                       *   overlay(text : 1)


     /**
      * \brief Parse JSON string
      *
      * Creates a JSON object from the passed string. If the passed
      * string is not a valid JSON string an escape message will be
      * sent. The string must be null terminated.
      *
      * \param Pointer to JSON string
      * \param Pointer to parser header (only for internal use)
      *
      * \return Pointer to JSON object
      */
     P json_parse      B                   export
     D                 PI              *
     D  string                         *   const options(*omit)
     D  ptrHeader                      *   const options(*nopass)
      *
     D json            S               *
     D key             S          10000A
      *
     D ptr             S               *
     D header          DS                  likeds(tmpl_parser_ds) based(ptr)
      *
     D retValDs        DS                  likeds(tmpl_return_value_ds)
     D dataType        S             10I 0
     D data            S              1A
     D keepLooping     S               N   inz(*on)
      /free
       json = json_create();

       if (%parms() = 2);
         // nested json object
         ptr = ptrHeader;
         data = '{';
       else;
         // first json object (top-level)
         ptr = %alloc(%size(tmpl_parser_ds));

         // init header ds
         header.json_string_ptr = string;
         header.index_pos = -1;

         data = nextCleanChar(ptr);
       endif;


       if (data = '{');

         dow (keepLooping);
           data = nextCleanChar(ptr);

           if (data = NULL);
             abortParsing(ptr : 'A JSON object text must end with "}"');
           elseif (data = '}');
             leave;
           else;
             // should be a key
             back(ptr);
             nextValue(%addr(retValDs) : ptr : dataType);
             if (dataType <> TYPE_STRING);
               abortParsing(ptr : 'Key is not of type string');
             endif;
             key = retValDs.text;

             // The key is followed by ':'. We will also tolerate '='.
             data = nextCleanChar(ptr);
             if (data <> ':' and data <> '=');
               abortParsing(ptr : 'Expected a ":" after a key');
             endif;

             // A value should be following
             nextValue(%addr(retValDs) : ptr : dataType);
             if (dataType = TYPE_STRING);
               json_putString(json : %trimr(key) : %trimr(retValDs.text));
             elseif (dataType = TYPE_BOOLEAN);
               json_putBoolean(json : %trimr(key) : retValDs.boolean);
             elseif (dataType = TYPE_NULL);
               json_putNull(json : %trimr(key));
             elseif (dataType = TYPE_INT);
               json_putInt(json : %trimr(key) : retValDs.integer);
             elseif (dataType = TYPE_LONG);
               json_putLong(json : %trimr(key) : retValDs.long);
             elseif (dataType = TYPE_DOUBLE);
               json_putDouble(json : %trimr(key) : retValDs.double);
             elseif (dataType = TYPE_OBJECT);
               json_putObject(json : %trimr(key) : retValDs.pointer);
             elseif (dataType = TYPE_ARRAY);
               json_putArray(json : %trimr(key) : retValDs.pointer);
             endif;

             // Pairs are separated by ','. We will also tolerate ';'.
             data = nextCleanChar(ptr);
             if (data = ',');
               // everything ok
             elseif (data = ';');
               // everything ok
             elseif (data = '}'); // end of object
               leave;
               abortParsing(ptr : 'Expected a "," or "}" after value');
             endif;

             // next iteration => next key/value pair
           endif;

         enddo;

       else;
         abortParsing(ptr : 'String is no JSON object string');
       endif;

       // deallocate memory
       if (%parms() = 1 and ptr <> *null);
         dealloc ptr;
       endif;

       return json;
      /end-free
     P                 E


     /**
      * \brief Next value
      *
      * Returns the next value. The parser is moved after the next value.
      * The returned value can be any supported JSON data type including array
      * and object.
      *
      * \param Pointer to the return value data structure
      * \param Pointer to the JSON string
      * \param JSON data type of the return value
      */
     P nextValue       B
     D                 PI
     D  retValPtr                      *   const
     D  ptr                            *
     D  dataType                     10I 0
      *
     D retValDs        DS                  likeds(tmpl_return_value_ds)
     D                                     based(retValPtr)
     D data            S              1A
     D backupData      S              1A
     D tmp             S          65535A
     D unquotedTextEnd...
     D                 S             13A   inz(',:]}/\''"[{;=#')
      *
     D STRING_TRUE     S              5A
     D STRING_FALSE    S              6A
     D STRING_NULL     S              5A
     D numbers         S             14A   inz('0123456789.,+-')
      /free
       STRING_TRUE = 'true' + NULL;
       STRING_FALSE = 'false' + NULL;
       STRING_NULL = 'null' + NULL;

       data = nextCleanChar(ptr);

       if (data = '"' or data = '''');
         // string
         retValDs.text = nextString(ptr : data);
         dataType = TYPE_STRING;
         return;

       elseif (data = '{');
         // json object
         retValDs.pointer = json_parse(*omit : ptr);
         dataType = TYPE_OBJECT;
         return;

       elseif (data = '[' or data = '(');
         // json array
         retValDs.pointer = nextArray(ptr);
         dataType = TYPE_ARRAY;
         return;

       elseif (data = ']' or data = ')');
         dataType = TYPE_NO_VALUE;
         return;

       elseif (data = '}');
         dataType = TYPE_NO_VALUE;
         return;

       else;
         //
         // Handle unquoted text. This could be the values true, false, or
         // null, or it can be a number. An implementation (such as this one)
         // is allowed to also accept non-standard forms.
         //
         // Accumulate characters until we reach the end of the text or a
         // formatting character.
         //
         backupData = data;
         dow (%scan(data : unquotedTextEnd) = 0);
           tmp = %trimr(tmp) + data;
           data = next(ptr);
         enddo;

         back(ptr);


         //
         // If it is true, false, or null, return the proper value.
         //

         // add null for string compare
         tmp = %trimr(tmp) + NULL;

         if (strcasecmp(%addr(tmp) : %addr(STRING_TRUE)) = 0);
           dataType = TYPE_BOOLEAN;
           retValDs.boolean = *on;
           return;
         elseif (strcasecmp(%addr(tmp) : %addr(STRING_FALSE)) = 0);
           dataType = TYPE_BOOLEAN;
           retValDs.boolean = *off;
           return;
         elseif (strcasecmp(%addr(tmp) : %addr(STRING_NULL)) = 0);
           dataType = TYPE_NULL;
           return;
         endif;


         // remove null from tmp
         tmp = %str(%addr(tmp));


         //
         // It should be a number
         //
         if (%scan(backupData : numbers) > 0);

           // determine if integer or double
           if (%scan('.' : tmp) > 0);
             // is double
             monitor;
               dataType = TYPE_DOUBLE;
               retValDs.double = %float(tmp);
               return;
               on-error *all;
                 // do nothing => value should be returned as string;
             endmon;
           else;
             monitor;
               dataType = TYPE_INT;
               retValDs.integer = %int(tmp);
               return;

               on-error 0103;

                 monitor;
                   dataType = TYPE_LONG;
                   retValDs.long = %int(tmp);
                   return;
                   on-error *all;
                     // do nothing => value should be returned as string
                     // because it ain't parsable as number
                 endmon;

               on-error 0202;
                 // do nothing => value should be returned as string
             endmon;
           endif;
         endif;

         //
         // Ain't a number => return value as string
         //
         dataType = TYPE_STRING;
         retValDs.text = tmp;

       endif;
      /end-free
     P                 E


     /**
      * \brief Next clean character
      *
      * Moves the parser to the position on the next clean character and
      * returns it to the caller. Character like carriage return, linefeed and
      * blanks are ignored. On a null character an escape message will be sent.
      *
      * \param Pointer to the parser header structure
      *
      * \return next clean character
      */
     P nextCleanChar   B
     D                 PI             1A
     D  ptr                            *
      *
     D header          DS                  likeds(tmpl_parser_ds) based(ptr)
     D data            S              1A
     D retVal          S              1A
     D keepLooping     S               N   inz(*on)
      /free
       dow (keepLooping);
         data = next(ptr);

         if (data = *blank);
           // loop
         elseif (data = CR);
           // loop
         elseif (data = LF);
           // loop
         elseif (data = NULL);
           abortParsing(ptr : 'Unexpected ending of JSON string');
         else;
           keepLooping = *off;
         endif;
       enddo;

       retVal = data;

       return retVal;
      /end-free
     P                 E

     /**
      * \brief Next string
      *
      * Returns the next string from the current parser position on and moves
      * the parser to the position behind the returned string.
      *
      * \param Pointer to the parser header structure
      * \param Open quote character of the string (' or ")
      *
      * \return next JSON string
      */
     P nextString      B
     D                 PI         10000A
     D   ptr                           *
     D   quote                        1A   const
      *
     D header          DS                  likeds(tmpl_parser_ds) based(ptr)
     D data            S              1A
     D keepLooping     S               N   inz(*on)
     D retVal          S          10000A   varying
      /free
       dow (keepLooping);
         data = next(ptr);

         if (data = CR);
           // throw esc message (unterminated end)
           abortParsing(ptr : 'Unterminated string (cr)');
         elseif (data = LF);
           // throw esc message (unterminated end)
           abortParsing(ptr : 'Unterminated string (linefeed)');
         elseif (data = NULL);
           // throw esc message (unterminated end)
           abortParsing(ptr : 'Unterminated string (null)');
         elseif (data = quote); // end of string
           keepLooping = *off;
         else;
           retVal += data;
         endif;

       enddo;

       return retVal;
      /end-free
     P                 E


     /**
      * \brief Next JSON array
      *
      * Returns the JSON array which is at the next position in the JSON
      * string. This procedure expects the position of the parser to be on
      * the opening bracket of the array ( [ ). If there is no JSON array
      * array at the current position an escape message will be sent.
      *
      * <br><br>
      *
      * Empty arrays are also supported.
      *
      * \param Pointer to the parser header structure
      *
      * \return Pointer to JSON array
      */
     P nextArray       B
     D                 PI              *
     D   ptr                           *
      *
     D header          DS                  likeds(tmpl_parser_ds) based(ptr)
     D array           S               *
     D retValDs        DS                  likeds(tmpl_return_value_ds)
     D dataType        S             10I 0
     D data            S              1A   inz
     D keepLooping     S               N   inz(*on)
      /free
       array = jsona_create();

       dow (keepLooping);

         // Values are separated by ','. We will also tolerate ';'.
         if (data = NULL);
           abortParsing(ptr : 'A JSON array text must end with "]"');
         elseif (data = ']');
           // end of array
           leave;
         elseif (data = ',');
           // everything ok
         elseif (data = ';');
           // everything ok
         endif;

         // arrays consists of values only
         nextValue(%addr(retValDs) : ptr : dataType);
         if (dataType = TYPE_STRING);
           jsona_putString(array : %trimr(retValDs.text));
         elseif (dataType = TYPE_BOOLEAN);
           jsona_putBoolean(array : retValDs.boolean);
         elseif (dataType = TYPE_NULL);
           jsona_putNull(array);
         elseif (dataType = TYPE_INT);
           jsona_putInt(array : retValDs.integer);
         elseif (dataType = TYPE_LONG);
           jsona_putLong(array : retValDs.long);
         elseif (dataType = TYPE_DOUBLE);
           jsona_putDouble(array : retValDs.double);
         elseif (dataType = TYPE_OBJECT);
           jsona_putObject(array : retValDs.pointer);
         elseif (dataType = TYPE_ARRAY);
           jsona_putArray(array : retValDs.pointer);
         elseif (dataType = TYPE_NO_VALUE);
           leave;
         endif;

         // get next char
         data = nextCleanChar(ptr);

         // next iteration

       enddo;

       return array;
      /end-free
     P                 E


     /**
      * \brief Next character
      *
      * Returns the next character in the JSON string and moves the parser
      * forward by one position.
      *
      * \param Pointer to parser header structure
      *
      * \return next character or null if there are no more characters
      */
     P next            B
     D                 PI             1A
     D  ptrHead                        *
      *
     D header          DS                  likeds(tmpl_parser_ds) based(ptrHead)
     D ptr             S               *
     D data            S              1A   based(ptr)
     D retVal          S              1A
      /free
       if (more(ptrHead));
         ptr = header.json_string_ptr + header.index_pos + 1;
         retVal = data;
         header.index_pos += 1;
       else;
         retVal = NULL;
       endif;

       return retVal;
      /end-free
     P                 E


     /**
      * \brief Move parser back
      *
      * Moves the position of the parser back by one position.
      *
      * \param Pointer to parser header structure
      */
     P back            B
     D                 PI
     D  ptr                            *
      *
     D header          DS                  likeds(tmpl_parser_ds) based(ptr)
      /free
       header.index_pos -= 1;
      /end-free
     P                 E


     /**
      * \brief Checks for more characters in string
      *
      * Checks if there are still more characters to parse in the given string.
      *
      * \param Pointer to the rest of the JSON string to be parsed
      */
     P more            B
     D                 PI              N
     D  ptr                            *
      /free
       if (strlen(ptr) = 0);
         return *off;
       else;
         return *on;
       endif;
      /end-free
     P                 E


     /**
      * \brief Abort parsing
      *
      * The memory of the parser header is deallocated (freed) and an
      * escape message is sent to the previous call stack entry with the
      * passed message text.
      *
      * \param Pointer to parser header
      * \param Message text
      */
     P abortParsing    B
     D                 PI
     D   ptr                           *
     D   message                   1000A   const
      *
     D sendProgramMessage...
     D                 PR                  extpgm('QMHSNDPM')
     D  szMsgID                       7A   const
     D  szMsgFile                    20A   const
     D  szMsgData                  6000A   const  options(*varsize)
     D  nMsgDataLen                  10I 0 const
     D  szMsgType                    10A   const
     D  szCallStkEntry...
     D                               10A   const
     D  nRelativeCallStkEntry...
     D                               10I 0 const
     D  szRtnMsgKey                   4A
     D  error                       265A   options(*varsize)
      *
     D msgdata         S            512A
     D msgkey          S              4A
     D apiError        S            265A
      /free
       // dispose header
       dealloc(n) ptr;

       // set message data
       msgdata = message;

       // send escape message
       sendProgramMessage('CPF9898' :
                          'QCPFMSG   *LIBL     ' :
                          %trimr(msgdata) :
                          %len(%trimr(msgdata)) :
                          '*ESCAPE   ' :
                          '*' :
                          1 :
                          msgkey :
                          apiError);
      /end-free
     P                 E
